@@ -5,7 +5,6 @@ from datetime import timedelta, datetime
 from math import ceil
 from logging import getLogger
 from pathlib import Path
-from xml.etree import ElementTree
 
 from numpy import arange, asarray, concatenate, empty, int64, zeros
 
@@ -240,16 +239,16 @@ class Dataset:
 
         Parameters
         ----------
-        begtime : int or datedelta or datetime or list
+        begtime : int or timedelta or datetime or list
             start of the data to read;
             if it's int, it's assumed it's s;
-            if it's datedelta, it's assumed from the start of the recording;
+            if it's timedelta, it's assumed from the start of the recording;
             if it's datetime, it's assumed it's absolute time.
             It can also be a list of any of the above type.
-        endtime : int or datedelta or datetime
+        endtime : int or timedelta or datetime
             end of the data to read;
             if it's int, it's assumed it's s;
-            if it's datedelta, it's assumed from the start of the recording;
+            if it's timedelta, it's assumed from the start of the recording;
             if it's datetime, it's assumed it's absolute time.
             It can also be a list of any of the above type.
 
@@ -291,20 +290,20 @@ class Dataset:
         return videos
 
     def read_data(self, chan=None, begtime=None, endtime=None, begsam=None,
-                  endsam=None, s_freq=None):
+                  endsam=None, events=None, pre=1, post=1, s_freq=None):
         """Read the data and creates a ChanTime instance
 
         Parameters
         ----------
         chan : list of strings
             names of the channels to read
-        begtime : int or datedelta or datetime or list
+        begtime : int or timedelta or datetime or list
             start of the data to read;
             if it's int or float, it's assumed it's s;
             if it's timedelta, it's assumed from the start of the recording;
             if it's datetime, it's assumed it's absolute time.
             It can also be a list of any of the above type.
-        endtime : int or datedelta or datetime
+        endtime : int or timedelta or datetime
             end of the data to read;
             if it's int or float, it's assumed it's s;
             if it's timedelta, it's assumed from the start of the recording;
@@ -314,6 +313,16 @@ class Dataset:
             first sample (this sample will be included)
         endsam : int
             last sample (this sample will NOT be included)
+        events : list of int or of timedelta or of datetime
+            list of the onset time of the events of interest.
+            This option is useful if you want to run a trial-based analysis.
+        pre : float
+            only when "events" is specified, the amount of data before each
+            event to be included (in s). Use a positive number to indicate
+            the time before the event.
+        post : float
+            only when "events" is specified, the amount of data after each
+            event to be included (in s).
         s_freq : int
             sampling frequency of the data
 
@@ -326,12 +335,16 @@ class Dataset:
         begsam and endsam follow Python convention, which starts at zero,
         includes begsam but DOES NOT include endsam.
 
-        If begtime and endtime are a list, they both need the exact same
+        If begtime and endtime are a list, the two lists should have the same
         length and the data will be stored in trials.
 
         If neither begtime or begsam are specified, it starts from the first
         sample. If neither endtime or endsam are specified, it reads until the
         end.
+
+        The time axis will indicate the time in seconds from data.start_time,
+        unless you specify "events". In that case, time will run from -"pre" to
+        +"post".
         """
         data = ChanTime()
         data.start_time = self.header['start_time']
@@ -352,23 +365,17 @@ class Dataset:
         if endtime is None and endsam is None:
             endsam = self.header['n_samples']
 
-        if begtime is not None:
-            if not isinstance(begtime, list):
-                begtime = [begtime]
-            begsam = []
-            for one_begtime in begtime:
-                begsam.append(_convert_time_to_sample(one_begtime, self))
-        if endtime is not None:
-            if not isinstance(endtime, list):
-                endtime = [endtime]
-            endsam = []
-            for one_endtime in endtime:
-                endsam.append(_convert_time_to_sample(one_endtime, self))
+        if events is not None:
+            eventssam = self._convert_to_list_with_samples(events)
+            presam = int(pre * s_freq)
+            postsam = int(post * s_freq)
+            begsam = [event - presam for event in eventssam]
+            endsam = [event + postsam for event in eventssam]
 
-        if not isinstance(begsam, list):
-            begsam = [begsam]
-        if not isinstance(endsam, list):
-            endsam = [endsam]
+            event_t = arange(-presam, postsam) / s_freq
+
+        begsam = self._convert_to_list_with_samples(begtime, begsam)
+        endsam = self._convert_to_list_with_samples(endtime, endsam)
 
         if len(begsam) != len(endsam):
             raise ValueError('There should be the same number of start and ' +
@@ -393,10 +400,27 @@ class Dataset:
 
             data.data[i] = dat
             data.axis['chan'][i] = asarray(chan_in_dat, dtype='U')
-            data.axis['time'][i] = (arange(one_begsam, one_endsam) / s_freq)
+            if events is not None:
+                data.axis['time'][i] = event_t
+            else:
+                data.axis['time'][i] = arange(one_begsam, one_endsam) / s_freq
 
         return data
 
+    def _convert_to_list_with_samples(self, times=None, samples=None):
+        """Convenience function to convert the input into a list of samples"""
+        if times is not None:
+            if not isinstance(times, list):
+                times = [times]
+            samples = []
+            for one_time in times:
+                samples.append(_convert_time_to_sample(one_time, self))
+
+        if not isinstance(samples, list):
+            samples = [samples]
+
+        return samples
+
 
 def _count_openephys_sessions(filename):
     """Open-ephys can have multiple sessions. We count how many files are in
