@@ -13,7 +13,7 @@ from math import ceil
 from re import search, match
 from xml.etree import ElementTree
 
-from numpy import array, empty, NaN, fromfile, unique, where
+from numpy import array, empty, NaN, fromfile, unique, where, arange, hstack, vstack, concatenate
 
 lg = getLogger(__name__)
 
@@ -28,6 +28,14 @@ BEG_BLK_SIZE = calcsize(BEG_BLK)
 DAT_FMT_SIZE = calcsize(DAT_FMT)
 BLK_SIZE = BEG_BLK_SIZE + DAT_FMT_SIZE + calcsize(END_BLK)
 
+EVENT_TYPES = {
+    3: 'TTL Event',
+    5: 'Network Event',
+    }
+IGNORE_EVENTS = [
+    'Network Event',
+    ]
+
 
 class OpenEphys:
     """Provide class OpenEphys, which can be used to read the folder
@@ -85,55 +93,49 @@ class OpenEphys:
         start_time = _read_date(self.settings_xml)
 
         self.recordings = _read_openephys(self.openephys_file)
-        s_freq = self.recordings[0]['s_freq']
-        self.s_freq = s_freq
+        self.s_freq = self.recordings[0]['s_freq']
         channels = self.recordings[0]['channels']
-
-        segments, messages = _read_messages_events(self.messages_file)
-        self.segments = segments
-        self.messages = messages
-        self.offsets = [int(x['channels'][0]['position']) for x in self.recordings]
-
-        for i in range(len(self.segments) - 1):
-            self.segments[i]['length'] = int((self.offsets[i + 1] - self.offsets[0]) / BLK_SIZE * BLK_LENGTH)
-            self.segments[i]['data_offset'] = self.offsets[i]
-
-        self.segments[-1]['data_offset'] = self.offsets[-1]
+        self.segments, self.messages, self.offset = _read_messages_events(self.messages_file)
 
         # only use channels that are actually in the folder
         chan_name = []
         self.channels = []
         gain = []
-        first_timestamps = []
         for chan in channels:
             channel_filename = (self.filename / chan['filename'])
 
             if channel_filename.exists():
                 chan_name.append(chan['name'])
                 self.channels.append(channel_filename)
-                ch_gain, ch_timestamp = _check_header(channel_filename, s_freq)
+
+                ch_gain = _check_header(channel_filename, self.s_freq, self.offset)
                 gain.append(ch_gain)
-                first_timestamps.append(ch_timestamp)
 
             else:
                 lg.warning(f'could not find {chan["filename"]} in {self.filename}')
         self.gain = array(gain)
 
+        # read data structure (recordings can have multiple segments)
+        data_offset = [int(x['channels'][0]['position']) for x in self.recordings]
+        for i in range(len(self.segments) - 1):
+            self.segments[i]['length'] = int((data_offset[i + 1] - data_offset[i]) / BLK_SIZE * BLK_LENGTH)
+            self.segments[i]['data_offset'] = data_offset[i]
+
+        self.segments[-1]['data_offset'] = data_offset[-1]
+
         file_length = channel_filename.stat().st_size
-        self.segments[-1]['length'] = int((file_length - self.offsets[-1]) / BLK_SIZE * BLK_LENGTH)
+        self.segments[-1]['length'] = int((file_length - data_offset[-1]) / BLK_SIZE * BLK_LENGTH)
 
         for seg in self.segments:
             seg['end'] = seg['start'] + seg['length']
 
-        # the first timestamp should be the same for all the channels
-        assert len(set(first_timestamps)) == 1
-        self.first_timestamp = first_timestamps[0]
+        n_samples = self.segments[-1]['end']
 
-        n_samples = self.segments[-1]['end'] + self.first_timestamp
+        self.blocks_dat, self.blocks_offset = _prepare_blocks(self.segments)
 
         orig = {}
 
-        return subj_id, start_time, s_freq, chan_name, n_samples, orig
+        return subj_id, start_time, self.s_freq, chan_name, n_samples, orig
 
     def return_dat(self, chan, begsam, endsam):
         """Read the data for some/all of the channels
@@ -152,21 +154,16 @@ class OpenEphys:
         2D array
             chan X samples recordings
         """
-        begsam -= self.first_timestamp
-        endsam -= self.first_timestamp
-
         data_length = endsam - begsam
         dat = empty((len(chan), data_length))
         dat.fill(NaN)
 
-        blocks_dat, blocks_offset = _prepare_blocks(self.segments)
-
-        all_blocks = _select_blocks(blocks_dat, begsam, endsam)
+        all_blocks = _select_blocks(self.blocks_dat, begsam, endsam)
         for i_chan, sel_chan in enumerate(chan):
             with self.channels[sel_chan].open('rb') as f:
                 for i_block in all_blocks:
-                    i_dat = blocks_dat[i_block, :] - begsam
-                    i_disk = blocks_offset[i_block].item()
+                    i_dat = self.blocks_dat[i_block, :] - begsam
+                    i_disk = self.blocks_offset[i_block].item()
 
                     # read only data (no timestamp or record marker)
                     f.seek(i_disk)
@@ -186,8 +183,8 @@ class OpenEphys:
         """
         all_markers = (
             self.messages
-            + _segments_to_markers(self.segments, self.first_timestamp)
-            + _read_all_channels_events(self.events_file, self.s_freq)
+            + _segments_to_markers(self.segments, self.offset)
+            + _read_all_channels_events(self.events_file, self.s_freq, self.offset)
             )
         return sorted(all_markers, key=lambda x: x['start'])
 
@@ -292,7 +289,7 @@ def _read_header(filename):
     return header, first_timestamp
 
 
-def _check_header(channel_file, s_freq):
+def _check_header(channel_file, s_freq, offset):
     """For each file, make sure that the header is consistent with the
     information in the text file.
 
@@ -302,6 +299,8 @@ def _check_header(channel_file, s_freq):
         path to single filename with the header
     s_freq : int
         sampling frequency
+    offset : int
+        offset of the first timestamp
 
     Returns
     -------
@@ -317,8 +316,9 @@ def _check_header(channel_file, s_freq):
     assert int(hdr['header_bytes']) == HDR_LENGTH
     assert int(hdr['blockLength']) == BLK_LENGTH
     assert int(hdr['sampleRate']) == s_freq
+    assert first_timestamp == offset
 
-    return float(hdr['bitVolts']), first_timestamp
+    return float(hdr['bitVolts'])
 
 
 def _segments_to_markers(segments, first_timestamp):
@@ -327,14 +327,14 @@ def _segments_to_markers(segments, first_timestamp):
         mrk.append({
             'name': f'START RECORDING #{i}',
             'chan': None,
-            'start': (seg['start'] + first_timestamp) / seg['s_freq'],
-            'end': (seg['start'] + first_timestamp) / seg['s_freq'],
+            'start': seg['start'] / seg['s_freq'],
+            'end': seg['start'] / seg['s_freq'],
         })
         mrk.append({
             'name': f'END RECORDING #{i}',
             'chan': None,
-            'start': (seg['end'] + first_timestamp) / seg['s_freq'],
-            'end': (seg['end'] + first_timestamp) / seg['s_freq'],
+            'start': seg['end'] / seg['s_freq'],
+            'end': seg['end'] / seg['s_freq'],
         })
 
     return mrk
@@ -352,6 +352,7 @@ def _read_messages_events(messages_file):
             m_event = match(r'(\d+) (.+)', l)
 
             if m_time:
+                # ignore Software time
                 pass
 
             elif m_start:
@@ -374,10 +375,10 @@ def _read_messages_events(messages_file):
                     'chan': None,
                 })
 
-    return segments, messages
+    return segments, messages, offset
 
 
-def _read_all_channels_events(events_file, s_freq):
+def _read_all_channels_events(events_file, s_freq, offset):
 
     file_read = [
         ('timestamps', '<i8'),
@@ -401,12 +402,15 @@ def _read_all_channels_events(events_file, s_freq):
 
         for i_on, i_off in zip(onsets, offsets):
             mrk.append({
-                'name': str(evt_type),
-                'start': i_on / s_freq,
-                'end': i_off / s_freq,
+                'name': EVENT_TYPES[evt_type],
+                'start': (i_on - offset) / s_freq,
+                'end': (i_off - offset) / s_freq,
                 'chan': None,
             })
 
+    # skip some events (like Network Events) which are not very useful
+    mrk = [evt for evt in mrk if not evt['name'] in IGNORE_EVENTS]
+
     return mrk
 
 
@@ -417,17 +421,16 @@ def _prepare_blocks(segments):
 
     for seg in segments:
         n_blocks = ceil(seg['length'] / BLK_LENGTH)
-        for i_blk in range(n_blocks):
-            blocks_dat.append([
-                i_blk * BLK_LENGTH + seg['start'],
-                i_blk * BLK_LENGTH + BLK_LENGTH + seg['start'],
-            ])
-            blocks_offset.append([
-                seg['data_offset'] + BEG_BLK_SIZE + i_blk * BLK_SIZE,
-            ])
 
-    blocks_dat = array(blocks_dat)
-    blocks_offset = array(blocks_offset)
+        blocks_dat.append(vstack((
+            arange(n_blocks) * BLK_LENGTH + seg['start'],
+            arange(n_blocks) * BLK_LENGTH + BLK_LENGTH + seg['start'],
+            )))
+        blocks_offset.append(
+            arange(n_blocks) * BLK_SIZE + seg['data_offset'] + BEG_BLK_SIZE)
+
+    blocks_dat = hstack(blocks_dat).T
+    blocks_offset = concatenate(blocks_offset)
 
     return blocks_dat, blocks_offset
 
