@@ -12,8 +12,9 @@ will be added as we need them.
 from collections.abc import Iterable
 from logging import getLogger
 
-from numpy import (arange, asarray, diff, empty, hstack, inf, linspace,
-                   nan_to_num, ones, ravel, setdiff1d, floor)
+from numpy import (arange, array, asarray, diff, empty, hstack, inf, 
+                   issubsctype, linspace, nan_to_num, ndarray, ones, ravel, 
+                   setdiff1d, floor)
 from numpy.lib.stride_tricks import as_strided
 from math import isclose
 from scipy.signal import resample as sci_resample
@@ -61,7 +62,7 @@ class Segments():
         return self.segments[index]
 
     def read_data(self, chan=[], ref_chan=[], grp_name=None, concat_chan=False,
-                  max_s_freq=30000, parent=None):
+                  average_channels=False, max_s_freq=30000, parent=None):
         """Read data for analysis. Adds data as 'data' in each dict.
 
         Parameters
@@ -76,6 +77,9 @@ class Segments():
             name of the channel group, required in GUI
         concat_chan : bool
             if True, data from all channels will be concatenated
+        average_channels : bool
+            if True, all channels will be averaged into a single virtual 
+            channel with label 'avg_chan'
         max_s_freq: : int
             maximum sampling frequency
         parent : QWidget
@@ -147,8 +151,14 @@ class Segments():
                     one_segment.data[0][i, :] = hstack(
                             [x(chan=ch)[0] for x in subseg])
 
+            if average_channels:
+                one_segment.data[0] = one_segment.data[0].mean(0, 
+                                                               keepdims=True)
+                one_segment.axis['chan'][0] = array(['avg_chan'], dtype='<U2')
+                active_chan = ['avg_chan']
+
             # For channel concatenation
-            if concat_chan and len(chs) > 1:
+            elif concat_chan and len(chs) > 1:
                 one_segment.data[0] = ravel(one_segment.data[0])
                 one_segment.axis['chan'][0] = asarray([(', ').join(chs)],
                                 dtype='U')
@@ -188,9 +198,9 @@ def select(data, trial=None, invert=False, **axes_to_select):
     **axes_to_select, optional
         Values need to be tuple or list. If the values in one axis are string,
         then you need to specify all the strings that you want. If the values
-        are numeric, then you should specify the range (you cannot specify
-        single values, nor multiple values). To select only up to one point,
-        you can use (None, value_of_interest)
+        are numeric, then you should specify the range. To select only up to one
+        point, you can use (None, value_of_interest). To select multiple
+        values, you can pass a numpy array with dtype bool
     invert : bool
         take the opposite selection
 
@@ -236,7 +246,9 @@ def select(data, trial=None, invert=False, **axes_to_select):
                     selected_values = asarray(values_to_select, dtype='U')
 
                 else:
-                    if (values_to_select[0] is None and
+                    if isinstance(values_to_select, ndarray) and issubsctype(values_to_select.dtype, bool):
+                        bool_values = values_to_select
+                    elif (values_to_select[0] is None and
                        values_to_select[1] is None):
                         bool_values = ones(len(values), dtype=bool)
                     elif values_to_select[0] is None:
