@@ -2,33 +2,38 @@
 """
 
 from logging import getLogger
+from fractions import Fraction
+from numpy import arange, array, zeros
+from scipy.signal import resample_poly
 
 from PyQt5.QtCore import Qt
 from PyQt5.QtWidgets import (QComboBox,
                              QErrorMessage,
                              QFormLayout,
+                             QGridLayout,
                              QGroupBox,
                              QHBoxLayout,
+                             QLabel,
                              QVBoxLayout,
                              )
 
 from ..detect import DetectSpindle, DetectSlowWave
-from ..trans import fetch, math
+from ..trans import fetch, math, filter_
 from .modal_widgets import ChannelDialog
 from .notes import SPINDLE_METHODS, SLOW_WAVE_METHODS
-from .utils import FormStr, FormFloat, FormBool, FormMenu
+from .utils import FormStr, FormFloat, FormBool, FormMenu, FormInt
 
 lg = getLogger(__name__)
 
 class SpindleDialog(ChannelDialog):
     """Dialog for specifying spindle detection parameters, ie:
-    label, channel, stage, lowcut, highcut, min dur, max dur, detection method,
+    name, channel, stage, lowcut, highcut, min dur, max dur, detection method,
     wavelet sigma, detection window, smoothing, detection threshold, selection
     threshold, minimum interval, merge across channels.
 
     Attributes
     ----------
-    label : str
+    name : str
         name of event type (to be added to or created)
     method : str
         name of detection method
@@ -47,15 +52,15 @@ class SpindleDialog(ChannelDialog):
         """ Create the dialog."""
         box0 = QGroupBox('Info')
 
-        self.label = FormStr()
+        self.name = FormStr()
 
-        self.label.setText('spin')
+        self.name.setText('spin')
         self.idx_group.activated.connect(self.update_channels)
         self.idx_chan.itemSelectionChanged.connect(self.count_channels)
 
         form = QFormLayout(box0)
-        form.addRow('Label',
-                       self.label)
+        form.addRow('Event name',
+                       self.name)
         form.addRow('Channel group',
                        self.idx_group)
         form.addRow('Channel(s)',
@@ -67,19 +72,25 @@ class SpindleDialog(ChannelDialog):
 
         box1 = QGroupBox('Parameters')
 
+        self.idx_method = FormMenu(SPINDLE_METHODS)
         self.index['f1'] = FormFloat()
         self.index['f2'] = FormFloat()
         self.index['rolloff'] = FormFloat()
+        self.index['tolerance'] = FormFloat()
         self.index['min_dur'] = FormFloat()
         self.index['max_dur'] = FormFloat()
-        self.index['sigma'] = FormFloat()
-        self.index['win_sz'] = FormFloat()
-        self.index['smooth'] = FormFloat()
-        self.index['det_thresh_lo'] = FormFloat()
-        self.index['det_thresh_hi'] = FormFloat()
-        self.index['sel_thresh'] = FormFloat()
         self.index['interval'] = FormFloat()
-        self.idx_method = FormMenu(SPINDLE_METHODS)
+        
+        self.index['0'] = FormFloat() # sigma
+        self.index['1'] = FormFloat() # win_sz
+        self.index['2'] = FormFloat() # smooth
+        self.index['3'] = FormFloat() # det_thresh
+        self.index['4'] = FormFloat() # det_thresh_hi
+        self.index['5'] = FormFloat() # sel_thresh
+        
+        self.label = []
+        for i in range(6):
+            self.label.append(QLabel(''))
 
         self.method = self.idx_method.currentText()
         self.idx_method.currentIndexChanged.connect(self.update_values)
@@ -93,40 +104,36 @@ class SpindleDialog(ChannelDialog):
                        self.index['f2'])
         form.addRow('Roll-off (Hz)',
                        self.index['rolloff'])
+        
+        form.addRow(self.label[0], self.index['0'])
+        form.addRow(self.label[1], self.index['1'])
+        form.addRow(self.label[2], self.index['2'])
+        form.addRow(self.label[3], self.index['3'])
+        form.addRow(self.label[4], self.index['4'])
+        form.addRow(self.label[5], self.index['5'])
+
+        form.addRow('Tolerance (sec)',
+                       self.index['tolerance'])
         form.addRow('Min. duration (sec)',
                        self.index['min_dur'])
-        form.addRow(' Max. duration (sec)',
+        form.addRow('Max. duration (sec)',
                        self.index['max_dur'])
-        form.addRow('Wavelet sigma',
-                       self.index['sigma'])
-        form.addRow('Detection window',
-                       self.index['win_sz'])
-        form.addRow('Smoothing',
-                       self.index['smooth'])
-        form.addRow('Detection threshold, low',
-                       self.index['det_thresh_lo'])
-        form.addRow('Detection threshold, high',
-                       self.index['det_thresh_hi'])
-        form.addRow('Selection threshold',
-                       self.index['sel_thresh'])
-        form.addRow('Min. interval',
+        form.addRow('Min. interval (sec)',
                        self.index['interval'])
-
+        
         box3 = QGroupBox('Options')
 
-        self.index['detrend'] = FormBool('Detrend (linear)')
-        self.index['merge'] = FormBool('Merge events across channels')
         self.index['excl_epoch'] = FormBool('Exclude Poor signal epochs')
         self.index['excl_event'] = FormMenu(['none', 'channel-specific', 
                                       'from any channel'])
         self.index['min_seg_dur'] = FormFloat(5)
+        self.index['merge'] = FormBool('Merge events across channels')
 
         self.index['excl_epoch'].set_value(True)
         self.index['merge'].setCheckState(Qt.Unchecked)
         self.index['merge'].setEnabled(False)
 
         form = QFormLayout(box3)
-        form.addRow(self.index['detrend'])
         form.addRow(self.index['excl_epoch'])
         form.addRow('Exclude Artefact events', 
                     self.index['excl_event'])
@@ -134,6 +141,32 @@ class SpindleDialog(ChannelDialog):
                        self.index['min_seg_dur'])
         form.addRow(self.index['merge'])
 
+        box4 = QGroupBox('Pre-processing')
+        
+        self.index['resample'] = FormBool('Resample (polyphase)')
+        self.index['rs_freq'] = FormInt()
+        self.index['prep_filt'] = FormBool('Bandpass filter')
+        self.index['prep_lc'] = FormFloat()
+        self.index['prep_hc'] = FormFloat()
+        self.index['detrend'] = FormBool('Detrend (linear)')
+        
+        rsbox = QHBoxLayout()
+        rsbox.addWidget(self.index['rs_freq'])
+        rsbox.addWidget(QLabel('Hz'))
+        
+        filtbox = QHBoxLayout()
+        filtbox.addWidget(self.index['prep_lc'])
+        filtbox.addWidget(QLabel('-'))
+        filtbox.addWidget(self.index['prep_hc'])
+        filtbox.addWidget(QLabel('Hz'))
+        
+        grid = QGridLayout(box4)
+        #grid.addWidget(self.index['resample'], 0, 0)
+        #grid.addLayout(rsbox, 0, 1)
+        grid.addWidget(self.index['prep_filt'], 1, 0)
+        grid.addLayout(filtbox, 1, 1)
+        grid.addWidget(self.index['detrend'], 2, 0, 1, 2)        
+        
         self.bbox.clicked.connect(self.button_clicked)
 
         btnlayout = QHBoxLayout()
@@ -143,6 +176,7 @@ class SpindleDialog(ChannelDialog):
         vlayout = QVBoxLayout()
         vlayout.addWidget(box1)
         vlayout.addWidget(box3)
+        vlayout.addWidget(box4)
         vlayout.addStretch(1)
         vlayout.addLayout(btnlayout)
 
@@ -168,7 +202,7 @@ class SpindleDialog(ChannelDialog):
             cycle = self.get_cycles()
             stage = self.idx_stage.selectedItems()
             params = {k: v.get_value() for k, v in self.index.items()}
-            label = self.label.get_value()
+            name = self.name.get_value()
 
             if None in [params['f1'], params['f2']]:
                 self.parent.statusBar().showMessage(
@@ -220,11 +254,44 @@ class SpindleDialog(ChannelDialog):
                 return
 
             data = data[0]['data']
+            
+# =============================================================================
+#             if params['resample']:                
+#                 rs_freq = params['rs_freq']
+#                 ratio = rs_freq / data.s_freq
+#                 up = Fraction(ratio).numerator
+#                 dn = Fraction(ratio).denominator
+#                 
+#                 rs_dat = zeros((data.number_of('chan')[0], 
+#                                 int(data.number_of('time')[0] * ratio)))
+#                 
+#                 for i in range(data.number_of('chan')[0]):                    
+#                     rs_dat[i, :] = resample_poly(data.data[0][i], up, dn)
+#                          
+#                 data.data[0] = rs_dat
+#                 data.s_freq = rs_freq
+#                 start_time = data.axis['time'][0][0]
+#                 end_time = data.axis['time'][0][-1]
+#                 data.axis['time'][0] = arange(start_time, end_time, 
+#                          1 / rs_freq)
+#                 lg.info(str(data.data.shape))
+#                 lg.info(str(data.data[0].shape))
+#                 lg.info(str(data.data[0][0].shape))
+#                 lg.info(str(data.axis['time'].shape))
+#                 lg.info(str(data.s_freq))
+# =============================================================================
+            
+            if params['prep_filt']:
+                low_cut = params['prep_lc']
+                high_cut = params['prep_hc']
+                data = filter_(data, axis='time', low_cut=low_cut, 
+                               high_cut=high_cut)
+            
             if params['detrend']:
                 data = math(data, operator_name='detrend', axis='time')            
             
             self.parent.notes.detect_events(data, self.method, 
-                                            params, label=label)
+                                            params, label=name)
 
             self.accept()
 
@@ -235,34 +302,165 @@ class SpindleDialog(ChannelDialog):
         """Update form values when detection method is selected."""
         self.method = self.idx_method.currentText()
         spin_det = DetectSpindle(method=self.method)
-
-        if self.method in ['Wamsley2012', 'UCSD']:
-            self.index['win_sz'].set_value(spin_det.det_wavelet['dur'])
-        elif self.method == 'Ray2015':
-            self.index['win_sz'].set_value(spin_det.zwin['dur'])
-        else:
-            self.index['win_sz'].set_value(spin_det.moving_rms['dur'])
-
+        
         self.index['f1'].set_value(spin_det.frequency[0])
         self.index['f2'].set_value(spin_det.frequency[1])
-        self.index['rolloff'].set_value(spin_det.rolloff)
+        self.index['tolerance'].set_value(spin_det.tolerance)
         self.index['min_dur'].set_value(spin_det.duration[0])
-        self.index['max_dur'].set_value(spin_det.duration[1])
-        self.index['sigma'].set_value(spin_det.det_wavelet['sd'])
-        self.index['smooth'].set_value(spin_det.smooth['dur'])
-        self.index['det_thresh_lo'].set_value(spin_det.det_thresh_lo)
-        self.index['det_thresh_hi'].set_value(spin_det.det_thresh_hi)
-        self.index['sel_thresh'].set_value(spin_det.sel_thresh)
+        self.index['max_dur'].set_value(spin_det.duration[1])        
         self.index['interval'].set_value(spin_det.min_interval)
 
-        for param in ['sigma', 'win_sz', 'det_thresh_lo', 'det_thresh_hi',
-                      'sel_thresh', 'smooth', 'rolloff']:
-            widg = self.index[param]
-            if widg.get_value() == 0:
-                widg.set_value('N/A')
-                widg.setEnabled(False)
+        if spin_det.rolloff:
+            self.index['rolloff'].set_value(spin_det.rolloff)
+            self.index['rolloff'].setEnabled(True)
+        else:
+            self.index['rolloff'].set_value('N/A')
+            self.index['rolloff'].setEnabled(False)
+
+        if self.method == 'Ferrarelli2007':
+            self.label[0].setText('Detection threshold')
+            self.label[1].setText('Selection threshold')
+            self.label[2].setText('')
+            self.label[3].setText('')
+            self.label[4].setText('')
+            self.label[5].setText('')
+
+            self.index['0'].set_value(spin_det.det_thresh)
+            self.index['1'].set_value(spin_det.sel_thresh)        
+        
+        if self.method == 'Nir2011':
+            self.label[0].setText('Gaussian smoothing sigma (sec)')
+            self.label[1].setText('Detection threshold (SD)')
+            self.label[2].setText('Selection threshold (SD)')
+            self.label[3].setText('')
+            self.label[4].setText('')
+            self.label[5].setText('')
+
+            self.index['0'].set_value(spin_det.smooth['dur'])
+            self.index['1'].set_value(spin_det.det_thresh)
+            self.index['2'].set_value(spin_det.sel_thresh)            
+        
+        if self.method == 'Moelle2011':
+            self.label[0].setText('RMS window length (sec)')
+            self.label[1].setText('Smoothing window length (sec)')
+            self.label[2].setText('Detection threshold (SD)')
+            self.label[3].setText('')
+            self.label[4].setText('')
+            self.label[5].setText('')
+
+            self.index['0'].set_value(spin_det.moving_rms['dur'])
+            self.index['1'].set_value(spin_det.smooth['dur'])
+            self.index['2'].set_value(spin_det.det_thresh)
+
+        if self.method == 'Wamsley2012':
+            self.label[0].setText('Wavelet window length (sec)')
+            self.label[1].setText('Wavelet sigma (sec)')
+            self.label[2].setText('Smoothing window length (sec)')
+            self.label[3].setText('Detection threshold')
+            self.label[4].setText('')
+            self.label[5].setText('')
+            
+            self.index['0'].set_value(spin_det.det_wavelet['dur'])
+            self.index['1'].set_value(spin_det.det_wavelet['sd'])
+            self.index['2'].set_value(spin_det.smooth['dur'])
+            self.index['3'].set_value(spin_det.det_thresh)
+
+        if self.method == 'Martin2013':
+            self.label[0].setText('RMS window length (sec)')
+            self.label[1].setText('RMS window step (sec)')
+            self.label[2].setText('Detection threshold (percentile)')
+            
+            self.index['0'].set_value(spin_det.moving_rms['dur'])
+            self.index['1'].set_value(spin_det.moving_rms['step'])
+            self.index['2'].set_value(spin_det.det_thresh)
+        
+        if self.method == 'Ray2015':
+            self.label[0].setText('Smoothing window length (sec)')
+            self.label[1].setText('z-score window length (sec)')
+            self.label[2].setText('Detection threshold (z)')
+            self.label[3].setText('Selection threshold (z)')
+            self.label[4].setText('')
+            self.label[5].setText('')
+            
+            self.index['0'].set_value(spin_det.smooth['dur'])
+            self.index['1'].set_value(spin_det.zscore['step'])
+            self.index['2'].set_value(spin_det.det_thresh)
+            self.index['3'].set_value(spin_det.sel_thresh)
+
+        if self.method == 'Lacourse2018':
+            self.label[0].setText('Window length (sec)')
+            self.label[1].setText('Window step (sec)')
+            self.label[2].setText('Absolute power threshold')
+            self.label[3].setText('Relative power threshold')
+            self.label[4].setText('Covariance threshold')
+            self.label[5].setText('Correlation threshold')
+            
+            self.index['0'].set_value(spin_det.windowing['dur'])
+            self.index['1'].set_value(spin_det.windowing['step'])
+            self.index['2'].set_value(spin_det.abs_pow_thresh)
+            self.index['3'].set_value(spin_det.rel_pow_thresh)
+            self.index['4'].set_value(spin_det.covar_thresh)
+            self.index['5'].set_value(spin_det.corr_thresh)
+
+        if self.method == 'FASST':
+            self.label[0].setText('Detection threshold (percentile)')
+            self.label[1].setText('Smoothing window length (sec)')
+            self.label[2].setText('')
+            self.label[3].setText('')
+            self.label[4].setText('')
+            self.label[5].setText('')
+
+            self.index['0'].set_value(spin_det.det_thresh)
+            self.index['1'].set_value(spin_det.smooth['dur'])
+            
+        if self.method == 'FASST2':
+            self.label[0].setText('Detection threshold (percentile)')
+            self.label[1].setText('RMS window length (sec)')
+            self.label[2].setText('Smoothing window length (sec)')
+            self.label[3].setText('')
+            self.label[4].setText('')
+            self.label[5].setText('')
+
+            self.index['0'].set_value(spin_det.det_thresh)
+            self.index['1'].set_value(spin_det.moving_rms['dur'])
+            self.index['2'].set_value(spin_det.smooth['dur'])
+            
+        if 'UCSD' in self.method:
+            self.label[0].setText('Wavelet duration (sec)')
+            self.label[1].setText('Wavelet width (sec)')
+            self.label[2].setText('Smoothing window length (sec)')
+            self.label[3].setText('Detection threshold (SD)')
+            self.label[4].setText('Selection threshold (SD)')
+            self.label[5].setText('')
+
+            self.index['0'].set_value(spin_det.det_wavelet['dur'])
+            self.index['1'].set_value(spin_det.det_wavelet['width'])
+            self.index['2'].set_value(spin_det.det_wavelet['win'])
+            self.index['3'].set_value(spin_det.det_thresh)
+            self.index['4'].set_value(spin_det.sel_thresh)
+
+        if 'Concordia' in self.method:
+            self.label[0].setText('RMS window length (sec)')
+            self.label[1].setText('Smoothing window length (sec)')
+            self.label[2].setText('Detection threshold, low (SD)')
+            self.label[3].setText('Detection threshold, high (SD)')
+            self.label[4].setText('Tolerance (sec)')
+            self.label[5].setText('Selection threshold (SD)')
+
+            self.index['0'].set_value(spin_det.moving_rms['dur'])
+            self.index['1'].set_value(spin_det.smooth['dur'])
+            self.index['2'].set_value(spin_det.det_thresh)
+            self.index['3'].set_value(spin_det.det_thresh_hi)
+            self.index['4'].set_value(spin_det.tolerance)
+            self.index['5'].set_value(spin_det.sel_thresh)
+        
+        for i, j in enumerate(self.label):
+            one_param = self.index[str(i)]
+            if j.text() == '':
+                one_param.set_value('')
+                one_param.setEnabled(False)
             else:
-                widg.setEnabled(True)
+                one_param.setEnabled(True)
 
     def count_channels(self):
         """If more than one channel selected, activate merge checkbox."""
@@ -280,13 +478,13 @@ class SpindleDialog(ChannelDialog):
 
 class SWDialog(ChannelDialog):
     """Dialog for specifying SW detection parameters, ie:
-    label, channel, stage, min dur, max dur, detection method, lowcut, highcut,
+    name, channel, stage, min dur, max dur, detection method, lowcut, highcut,
     minimum and maximum trough duration, maximum trough amplitude, minimum
     peak-to-peak amplitude.
 
     Attributes
     ----------
-    label : str
+    name : str
         name of event type (to be added to or created)
     method : str
         name of detection method
@@ -304,14 +502,14 @@ class SWDialog(ChannelDialog):
         """ Create the dialog."""
         box0 = QGroupBox('Info')
 
-        self.label = FormStr()
+        self.name = FormStr()
 
-        self.label.setText('sw')
+        self.name.setText('sw')
         self.idx_group.activated.connect(self.update_channels)
 
         form = QFormLayout(box0)
-        form.addRow('Label',
-                           self.label)
+        form.addRow('Event name',
+                           self.name)
         form.addRow('Channel group',
                        self.idx_group)
         form.addRow('Channel(s)',
@@ -414,7 +612,7 @@ class SWDialog(ChannelDialog):
             cycle = self.get_cycles()
             stage = self.idx_stage.selectedItems()
             params = {k: v.get_value() for k, v in self.index.items()}
-            label = self.label.get_value()
+            name = self.name.get_value()
 
             if None in [params['f1'], params['f2']]:
                 self.parent.statusBar().showMessage(
@@ -463,7 +661,7 @@ class SWDialog(ChannelDialog):
                 data = math(data, operator_name='detrend', axis='time')
             
             self.parent.notes.detect_events(data, self.method, 
-                                            params, label=label)
+                                            params, label=name)
 
             self.accept()
 
